@@ -70,12 +70,16 @@ class TypeCollector(cst.CSTVisitor):
     def visit_FunctionDef(self, node: cst.FunctionDef) -> bool:
         self.qualifier.append(node.name.value)
         returns = node.returns
-        if returns is not None:
-            return_annotation = self._create_import_from_annotation(returns)
-            parameter_annotations = self._import_parameter_annotations(node.params)
-            self.function_annotations[".".join(self.qualifier)] = FunctionAnnotation(
-                parameters=parameter_annotations, returns=return_annotation
-            )
+        return_annotation = (
+            self._create_import_from_annotation(returns)
+            if returns is not None
+            else None
+        )
+        parameter_annotations = self._import_parameter_annotations(node.params)
+        self.function_annotations[".".join(self.qualifier)] = FunctionAnnotation(
+            parameters=parameter_annotations, returns=return_annotation
+        )
+
         # pyi files don't support inner functions, return False to stop the traversal.
         return False
 
@@ -462,7 +466,10 @@ class ApplyTypeAnnotationsVisitor(ContextAwareTransformer):
             function_annotation = self.annotations.function_annotations[key]
             # Only add new annotation if explicitly told to overwrite existing
             # annotations or if one doesn't already exist.
-            if self.overwrite_existing_annotations or not updated_node.returns:
+            set_return_annotation = not updated_node.returns or (
+                self.overwrite_existing_annotations and function_annotation.returns
+            )
+            if set_return_annotation:
                 updated_node = updated_node.with_changes(
                     returns=function_annotation.returns
                 )
