@@ -18,6 +18,76 @@ from libcst.testing.utils import data_provider
 class TestApplyAnnotationsVisitor(CodemodTest):
     TRANSFORM: Type[Codemod] = ApplyTypeAnnotationsVisitor
 
+    @data_provider(
+        (
+            (
+                """
+                from __future__ import annotations
+                from foo import Foo
+                from baz import Baz
+                """,
+                """
+                from foo import Bar
+                import bar
+                """,
+                """
+                from __future__ import annotations
+                from foo import Foo, Bar
+                import bar
+                from baz import Baz
+                """,
+            ),
+            (
+                # Missing feature: ignore aliased imports
+                """
+                from Foo import foo as bar
+                """,
+                """
+                from Foo import bar
+                """,
+                """
+                from Foo import bar
+                """,
+            ),
+            (
+                # Missing feature: ignore bare imports
+                """
+                import foo
+                """,
+                """
+                """,
+                """
+                """,
+            ),
+            (
+                # Missing feature: ignore relative imports
+                """
+                from .. import foo
+                """,
+                """
+                """,
+                """
+                """,
+            ),
+            (
+                # Missing feature: ignore star imports
+                """
+                from foo import *
+                """,
+                """
+                """,
+                """
+                """,
+            ),
+        )
+    )
+    def test_merge_module_imports(self, stub: str, before: str, after: str) -> None:
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context, parse_module(textwrap.dedent(stub.rstrip()))
+        )
+        self.assertCodemod(before, after, context_override=context)
+
     @data_provider(
         (
             (
@@ -608,22 +678,6 @@ class TestApplyAnnotationsVisitor(CodemodTest):
                     pass
                 """,
             ),
-            # Sanity check that we don't fail when the stub has relative imports.
-            # We don't do anything with those imports, though.
-            (
-                """
-                from .. import hello
-                def foo() -> typing.Sequence[int]: ...
-                """,
-                """
-                def foo():
-                  return []
-                """,
-                """
-                def foo() -> typing.Sequence[int]:
-                  return []
-                """,
-            ),
             (
                 """
                 from typing import Dict
@@ -669,6 +723,31 @@ class TestApplyAnnotationsVisitor(CodemodTest):
                     def foo(self, atticus, b: Optional[int] = None, c: bool = False): ...
                 """,
             ),
+            # Make sure we handle string annotations well
+            (
+                """
+                def f(x: "typing.Union[int, str]") -> "typing.Union[int, str]": ...
+
+                class A:
+                    def f(self: "A") -> "A": ...
+                """,
+                """
+                def f(x):
+                    return x
+
+                class A:
+                    def f(self):
+                        return self
+                """,
+                """
+                def f(x: "typing.Union[int, str]") -> "typing.Union[int, str]":
+                    return x
+
+                class A:
+                    def f(self: "A") -> "A":
+                        return self
+                """,
+            ),
         )
     )
     def test_annotate_functions(self, stub: str, before: str, after: str) -> None:
@@ -765,3 +844,84 @@ class TestApplyAnnotationsVisitor(CodemodTest):
             overwrite_existing_annotations=True,
         )
         self.assertCodemod(before, after, context_override=context)
+
+    @data_provider(
+        (
+            (
+                """
+                def fully_annotated_with_untyped_stub(a, b): ...
+                """,
+                """
+                def fully_annotated_with_untyped_stub(a: bool, b: bool) -> str:
+                    return "hello"
+                """,
+                """
+                def fully_annotated_with_untyped_stub(a: bool, b: bool) -> str:
+                    return "hello"
+                """,
+            ),
+            (
+                """
+                def params_annotated_with_return_from_stub(a, b) -> str: ...
+                """,
+                """
+                def params_annotated_with_return_from_stub(a: bool, b: bool):
+                    return "hello"
+                """,
+                """
+                def params_annotated_with_return_from_stub(a: bool, b: bool) -> str:
+                    return "hello"
+                """,
+            ),
+            (
+                """
+                def partially_annotated_params_with_partial_stub(a, b: int): ...
+                """,
+                """
+                def partially_annotated_params_with_partial_stub(a: bool, b) -> str:
+                    return "hello"
+                """,
+                """
+                def partially_annotated_params_with_partial_stub(a: bool, b: int) -> str:
+                    return "hello"
+                """,
+            ),
+            (
+                """
+                def async_with_decorators(a: bool, b: bool) -> str: ...
+                """,
+                """
+                @second_decorator
+                @first_decorator(5)
+                async def async_with_decorators(a, b):
+                    return "hello"
+                """,
+                """
+                @second_decorator
+                @first_decorator(5)
+                async def async_with_decorators(a: bool, b: bool) -> str:
+                    return "hello"
+                """,
+            ),
+        )
+    )
+    def test_annotate_using_incomplete_stubs(
+        self, stub: str, before: str, after: str
+    ) -> None:
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context, parse_module(textwrap.dedent(stub.rstrip()))
+        )
+        # Test setting the overwrite flag on the codemod instance.
+        self.assertCodemod(
+            before, after, context_override=context, overwrite_existing_annotations=True
+        )
+
+        # Test setting the flag when storing the stub in the context.
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context,
+            parse_module(textwrap.dedent(stub.rstrip())),
+            overwrite_existing_annotations=True,
+        )
+        self.assertCodemod(before, after, context_override=context)
