@@ -6,7 +6,7 @@
 import libcst as cst
 import libcst.matchers as m
 from libcst.codemod import CodemodContext, CodemodTest, VisitorBasedCodemodCommand
-from libcst.codemod.visitors import RemoveImportsVisitor
+from libcst.codemod.visitors import AddImportsVisitor, RemoveImportsVisitor
 from libcst.metadata import (
     QualifiedName,
     QualifiedNameProvider,
@@ -80,6 +80,12 @@ class TestRemoveImportsCodemod(CodemodTest):
                 short,
                 this_stays_too
             )
+            from fourth import (
+                a,
+                # comment
+                b,
+                c
+            )
         """
         after = """
             from foo import (
@@ -92,6 +98,10 @@ class TestRemoveImportsCodemod(CodemodTest):
             from third import (
                 this_stays_too
             )
+            from fourth import (
+                a,
+                c
+            )
         """
         self.assertCodemod(
             before,
@@ -101,6 +111,7 @@ class TestRemoveImportsCodemod(CodemodTest):
                 ("loooong", "short", None),
                 ("loooong", "bar", None),
                 ("third", "short", None),
+                ("fourth", "b", None),
             ],
         )
 
@@ -887,6 +898,21 @@ class TestRemoveImportsCodemod(CodemodTest):
             RemoveImportTransformer(CodemodContext()).transform_module(module).code,
         )
 
+    def test_remove_import_alias_after_inserting(self) -> None:
+        before = "from foo import bar, baz"
+        after = "from foo import quux, baz"
+
+        class AddRemoveTransformer(VisitorBasedCodemodCommand):
+            def visit_Module(self, node: cst.Module) -> None:
+                AddImportsVisitor.add_needed_import(self.context, "foo", "quux")
+                RemoveImportsVisitor.remove_unused_import(self.context, "foo", "bar")
+
+        module = cst.parse_module(self.make_fixture_data(before))
+        self.assertCodeEqual(
+            AddRemoveTransformer(CodemodContext()).transform_module(module).code,
+            after,
+        )
+
     def test_remove_comma(self) -> None:
         """
         Trailing commas should be removed if and only if the last alias is removed.
